@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import abc
+import datetime
 import uuid
 
 from ckan import model
@@ -8,8 +9,6 @@ from ckan.plugins import toolkit as tk
 
 from ckanext.tables import table, types
 
-formatter_registry: types.Registry[str, type[BaseFormatter]] = types.Registry({})
-
 
 class BaseFormatter(abc.ABC):
     """Abstract base class for all formatters."""
@@ -18,10 +17,12 @@ class BaseFormatter(abc.ABC):
         self,
         column: table.ColumnDefinition,
         row: types.Row,
+        initial_row: types.Row,
         table: table.TableDefinition,
     ):
         self.column = column
         self.row = row
+        self.initial_row = initial_row
         self.table = table
 
     @abc.abstractmethod
@@ -30,18 +31,34 @@ class BaseFormatter(abc.ABC):
 
 
 class DateFormatter(BaseFormatter):
-    """Formats a datetime string into a more readable date.
+    """Formats a datetime object into a more readable date.
 
     Options:
         - `date_format` (str): The strftime format for the output.
           Defaults to "%d/%m/%Y - %H:%M".
     """
 
-    def format(self, value: types.Value, options: types.Options) -> types.FormatterResult:
+    def format(self, value: datetime.datetime, options: types.Options) -> types.FormatterResult:
         date_format = options.get("date_format", "%d/%m/%Y - %H:%M")
         return tk.h.render_datetime(value, date_format=date_format)
 
 
+class URLFormatter(BaseFormatter):
+    """Generates a clickable link for a URL.
+
+    Options:
+        - `target` (str): The target attribute for the link. Defaults to "_blank".
+    """
+
+    def format(self, value: types.Value, options: types.Options) -> types.FormatterResult:
+        if not value:
+            return ""
+
+        target = options.get("target", "_blank")
+
+        return tk.literal(f"<a href='{value}' target='{target}'>{value}</a>")
+
+
 class UserLinkFormatter(BaseFormatter):
     """Generates a link to a user's profile with a placeholder avatar.
 
@@ -169,6 +186,8 @@ class DialogModalFormatter(BaseFormatter):
           Defaults to `tables/formatters/dialog_modal.html`.
         - `modal_title` (str): The title of the modal dialog.
           Defaults to "Details".
+        - `max_length` (int): The maximum length of the preview text before
+          truncation. Defaults to 100.
     """
 
     def format(self, value: types.Value, options: types.Options) -> types.FormatterResult:
@@ -176,6 +195,7 @@ class DialogModalFormatter(BaseFormatter):
             return ""
 
         template = options.get("template", "tables/formatters/dialog_modal.html")
+        max_length = options.get("max_length", 100)
         modal_title = options.get("modal_title", "Details")
 
         return tk.literal(
@@ -187,6 +207,7 @@ class DialogModalFormatter(BaseFormatter):
                     "column": self.column,
                     "row": self.row,
                     "modal_title": modal_title,
+                    "max_length": max_length,
                     "modal_id": f"modal-{uuid.uuid4().hex}",
                 },
             )
